# This code in R estimates the population growth rate of aphids and parasitoids as described in "Temperature effects on long-term population dynamics in a parasitoid-host system" by Matthew H. Meisner, Jason P. Harmon, and Anthony R. Ives. The data come from 3 years (1995-97) of field samples from alfalfa fields. The statistical model is given in Equation 1.

# Required package
# nlme

# Required data
# MeisnerFieldData.txt

library(nlme)

# MeisnerFieldData.txt

d <- read.table(file.choose(),sep = "\t",header=T)

d$field <- as.factor(1000*(d$year-1995) + d$field)
d$year <- as.factor(d$year)

attach(d)
dd <- d[!is.na(aphid_sweeps),]
c(mean(dd$aphid_sweeps),mean(dd$MeanTemp))

dd <- d[!is.na(aphid_sweeps) & !is.na(aphid_sweeps_tp1) & (day_tp1 - day)<=12,]
c(mean(dd$aphid_sweeps),mean(dd$MeanTemp))

dd <- d[!is.na(aphid_sweeps) & !is.na(avepara_tp1) & !is.na(avepara) & (day_tp1 - day)<=12,]
c(mean(dd$aphid_sweeps),mean(dd$MeanTemp))

# [1] 29.16979 19.92946
# [1] 31.06081 20.48907
# [1] 39.61307 20.34248

##############################################################################
# APHID POPULATION GROWTH
##############################################################################

dd <- d

dd$aphid_sweeps_tp1[dd$aphid_sweeps_tp1 == 0] <- 
min(dd$aphid_sweeps_tp1[dd$aphid_sweeps_tp1>0])/2
dd$aphid_sweeps[dd$aphid_sweeps == 0] <- min(dd$aphid_sweeps[dd$aphid_sweeps>0])/2

dd$avepara_tp1[dd$avepara_tp1 == 0] <- min(dd$avepara_tp1[dd$avepara_tp1>0])/2
dd$avepara[dd$avepara == 0] <- min(dd$avepara[dd$avepara>0])/2

dd$globalparasitoids[dd$globalparasitoids == 0] <- 
min(dd$globalparasitoids[dd$globalparasitoids>0])/2
dd$globalparasitoids_tp1[dd$globalparasitoids_tp1 == 0] <- 
min(dd$globalparasitoids_tp1[dd$globalparasitoids_tp1>0])/2

dd$aveparasitoids <- dd$avepara*dd$aphid_sweeps
dd$aveparasitoids_tp1 <- dd$avepara_tp1*dd$aphid_sweeps_tp1

ddd <- dd[is.na(dd$globalparasitoids_tp1)==F,]
ddd <- ddd[is.na(ddd$aphid_sweeps_tp1)==F,]
ddd <- ddd[is.na(ddd$globalaphids_tp1)==F,]
ddd <- ddd[is.na(ddd$aphid_sweeps)==F,]

ddd <- ddd[(ddd$day_tp1 - ddd$day) <= 12,]

summary(nlme(log(aphid_sweeps_tp1) ~ c*harvest + log(aphid_sweeps) + (day_tp1-
day)*(exp(r0+r1*MeanTemp)*(1-k*log(aphid_sweeps)) - exp(a0 + 
a1*MeanTemp)*globalparasitoids_tp1/globalaphids_tp1), 
		   start = c(c=.1,r0=.001,r1=.01,k=.1,a0=1,a1=.1), 
		   fixed = c + r0 + r1 + k + a0 + a1 ~ 1,
		   random = r0 ~ 1 | field,
		   correlation = corAR1(.1, form = ~1 | field),
		   data=ddd))

# Nonlinear mixed-effects model fit by maximum likelihood
  # Model: log(aphid_sweeps_tp1) ~ c * harvest + log(aphid_sweeps) + (day_tp1 -      day) * (exp(r0 + r1 * MeanTemp) * (1 - k * log(aphid_sweeps)) -      exp(a0 + a1 * MeanTemp) * globalparasitoids_tp1/(0 + globalaphids_tp1)) 
 # Data: ddd 
      # AIC      BIC    logLik
  # 567.969 595.9226 -274.9845

# Random effects:
 # Formula: r0 ~ 1 | field
                  # r0 Residual
# StdDev: 1.287316e-05  1.28217

# Correlation Structure: AR(1)
 # Formula: ~1 | field 
 # Parameter estimate(s):
        # Phi 
# -0.04738539 
# Fixed effects: c + r0 + r1 + k + a0 + a1 ~ 1 
       # Value Std.Error  DF   t-value p-value
# c  -2.399000 0.5737572 130 -4.181212  0.0001
# r0 -2.129319 0.5321204 130 -4.001574  0.0001
# r1  0.057186 0.0259655 130  2.202380  0.0294
# k   0.213148 0.0179922 130 11.846661  0.0000
# a0 -4.500226 1.0247979 130 -4.391330  0.0000
# a1  0.191030 0.0451650 130  4.229605  0.0000
 # Correlation: 
   # c      r0     r1     k      a0    
# r0 -0.042                            
# r1  0.044 -0.978                     
# k   0.283  0.071 -0.148              
# a0  0.013  0.564 -0.479 -0.127       
# a1 -0.014 -0.592  0.527  0.043 -0.991

# Standardized Within-Group Residuals:
       # Min         Q1        Med         Q3        Max 
# -4.0910939 -0.5455989  0.2133024  0.5779496  2.3937591 

# Number of Observations: 165
# Number of Groups: 30 
