% MeisnerLongtermCageFitting.m
% 
% Tony Ives, 15Dec13
% 
% Fits the data from the long-term cage experiments presented in Meisner et al., Ecological Monographs,
% in press.
% 
% Requires
% 
% Code
%	meisnerfigfunct_20Aug13.m (contains the Kalman filter
%	anneal.m written by Joachim Vandekerckhove (or similar simulated annealing function)
% 
% Data
%	MeisnerLongtermCageData.txt

global Y Tsample ib jb mb nb counter tempest ...
	Temp atemp SADdist stageInts mumInts year relatt sexratio L sj totStage timescale Rep ...
	flist

for superi=1
timescale=1;

figflag=1;

load 'MeisnerLongtermCageData.txt' -ascii
d=MeisnerLongtermCageData;
%1 year	
%2 day	
%3 treatment	
%4 replicate	
%5 mummies	
%6 aphids

% 2008
% For 2008, aphids were added on 8/17, and the first sample was 8/18.  
% Parasitoids were added the same day.  Then, it was sampled every 2-3 days (8/18, 8/20, etc...). 

% 2009
% On 6/5, plants marked with X were innoculated with 10 aphids of random ages
% Plants replaced if aphid total had dropped 25% or more since the last count 
% or had dropped less than 25% for 2 consecutive counts.
% 5 wasps added to each cage on 7/10 and 7/15
% first aphid/mum sample of 6/8

figure(1)
set(1,'position',[489   293   512   384])

trtcol='br';
for i=1:4
	subplot(2,2,1)
	dd=d(find(d(:,4)==i),:);
	plot(dd(:,2),dd(:,6),trtcol(dd(1,3)+1))
	ylabel('aphids')
	axis([0 80 0 1500])
	hold on
	
	subplot(2,2,3)
	dd=d(find(d(:,4)==i),:);
	plot(dd(:,2),dd(:,5),trtcol(dd(1,3)+1))
	ylabel('mummies')
	axis([0 80 0 150])
	hold on
end
	
for i=5:8
	subplot(2,2,2)
	dd=d(find(d(:,4)==i),:);
	plot(dd(:,2),dd(:,6),trtcol(dd(1,3)+1))
	axis([0 80 0 1500])
	ylabel('aphids')
	hold on
	
	subplot(2,2,4)
	dd=d(find(d(:,4)==i),:);
	plot(dd(:,2),dd(:,5),trtcol(dd(1,3)+1))
	axis([0 80 0 150])
	ylabel('mummies')
	hold on
end
	
% aphid growth rate data from PA tempdep Reproduction 28Aug09.xls

reproC=[0.906779661	3.903508772	5.422727273	5.537037037	5.663265306	5.597560976	6.138157895	...
	4.614285714	4.770114943	4.696969697	3.649122807	2	0.727272727	0.888888889	0.833333333	0.4	0	0	0	0	0	0	0];
reproH=[1.289285714	5.928571429	6.076086957	6.916666667	6.902985075	6.24609375	6.954545455	...
	5.990384615	6.19	4.183673469	4.321428571	3.468468468	2.416666667	2.493333333	1.175	0.966666667	0.5	1.2	0	0.5	0	0	0];
adultSurvC=[59	57	55	54	49	41	38	35	29	22	19	14	11	9	6	5	1	1	0	0	0	0	0];
adultSurvH=[70	70	69	69	67	64	55	52	50	49	42	37	32	25	20	15	12	10	7	6	3	2	1];

reproC=reproC(1:20);
reproH=reproH(1:20);
adultSurvC=adultSurvC(1:20);
adultSurvH=adultSurvH(1:20);

repro=[reproC;reproH];
repro=[repro [0;0]];

stageLengthC=[1.513888889	1.705882353	1.597014925	1.777777778];
stageLengthH=[1.328947368	1.472972973	1.369863014	1.577464789];
stageLength=[stageLengthC;stageLengthH];

% juvenile stage-specific survivorships
stagesurvC=[72	68	67	63];
stagesurvH=[76	74	73	71];

% stagesurvC(2:end)./stagesurvC(1:end-1)
% stagesurvH(2:end)./stagesurvH(1:end-1)

sj(1)=mean(log(stagesurvC(2:end)./stagesurvC(1:end-1))./stageLengthC(2:end));
sj(2)=mean(log(stagesurvH(2:end)./stagesurvH(1:end-1))./stageLengthH(2:end));

% convert to timescale d time scale
stageLength=timescale*round(stageLength/timescale);
sj=exp(timescale*sj);

% adult age-specific survivorships
adultSurvC=adultSurvC(1:20);
adultSurvH=adultSurvH(1:20);
adultSurv=[adultSurvC;adultSurvH];

sa=[];
sa(1,1:19)=adultSurvC(2:end)./adultSurvC(1:end-1);
sa(2,1:19)=adultSurvH(2:end)./adultSurvH(1:end-1);

sa(find(isnan(sa)))=0;
sa(find(isinf(sa)))=0;
sa(:,20)=0;

sa=[[1;1] sa [0;0]];
repro=[[0;0] repro];

% interpolate adult surv and repro
satemp=zeros(2,200);
reprotemp=zeros(2,200);

tstep1=2.5;
Tstep1=0:tstep1:21;
tstep2=timescale;
Tstep2=0:tstep2:21;

for i=1:2
	saInterpol1=interp1(0:21,sa(i,:),Tstep1,'cubic','extrap');
	saInterpol1(find(saInterpol1<0))=0;
	saInterpol1(end)=0;
	
	saInterpol=interp1(Tstep1,saInterpol1,Tstep2,'cubic','extrap');
	saInterpol(find(saInterpol<0))=0;
	
	if figflag
		figure(100+i)
		subplot(2,1,1)
		plot(0:21,sa(i,:),'.',Tstep2,saInterpol,'-')
	end
	
	reproInterpol1=interp1(0:21,repro(i,:),Tstep1,'cubic','extrap');
	reproInterpol1(find(reproInterpol1<0))=0;
	
	reproInterpol=interp1(Tstep1,reproInterpol1,Tstep2,'cubic','extrap');

	if figflag
		subplot(2,1,2)
		plot(0:21,repro(i,:),'.',Tstep2,reproInterpol,'-')
	end
	
	satemp(i,1:length(saInterpol))=saInterpol;
	reprotemp(i,1:length(reproInterpol))=reproInterpol;
end
satemp(:,end)=0;
reprotemp(:,end)=0;

satemp(satemp<0)=0;
reprotemp(reprotemp<0)=0;

sa=satemp.^timescale;
repro=timescale*reprotemp;

zz=find(repro(1,:)==0);
reproPeriod(1)=min(zz(2:end));

zz=find(repro(2,:)==0);
reproPeriod(2)=min(zz(2:end));

stageInts=round(stageLength/timescale);
stageInts=[stageInts reproPeriod'];

totStage=max(sum(stageInts'));
	
% from Ives et al 1999
relatt=[0.12 0.27 0.39 0.16 0.06];

relattemp=zeros(2,totStage);
for j=1:2
	counter=0;
	for i=1:5
		relattemp(j,counter+1:counter+stageInts(j,i))=relatt(i)*ones(1,stageInts(j,i));
		counter=counter+stageInts(j,i);
	end
end
relattbase=relattemp;

% mummy development times
%		E-M				E-A
% Hot	8.072815534		11.08938547
% Cold	10.06716418		18.1

mumIntsbase=floor([9.07 8.03;7.07 3.02]/timescale);

sexratio=.5;

Tsample=round(d(:,2)/timescale);
Temp=d(:,3);
year=d(:,1);
Rep=d(:,4);

Y=d(:,[6 5]);
Y(find(Y==0))=.5;
Y=log(Y);

%1 year	
%2 day	
%3 treatment	
%4 replicate	
%5 mummies	
%6 aphids

% temp-dependent parasitism
% run 2
% Cold	4.555555556
% Hot	7.466666667

% run 3
% Cold	7.4
% Hot	14.36842105

atemp=[.5*(4.56/(4.56+7.47) + 7.4/(7.4+14.37))];
atempbase=[atemp 1-atemp];

Flist=[];
LLlist=[];

% 1=aphid juv, 2=aphid adult, 3=para juv, 4=attack rate
tempset=[1 1 1 1];

if tempset(1)==1
	t1C=1;
	t1H=2;
	relatt=relattbase;
else
	t1C=2;
	t1H=1;
	relatt=flipud(relattbase);
end
if tempset(2)==1
	t2C=1;
	t2H=2;
else
	t2C=2;
	t2H=1;
end

L=zeros(totStage,totStage,2);
juvTime=sum(stageInts(t1C,1:end-1));
LL=diag([sj(t1C)*ones(1,juvTime) sa(t2C,1:totStage-juvTime-1)],-1);
LL(1,juvTime+1:juvTime+stageInts(t1C,end))=repro(t2C,1:stageInts(t1C,end));
L(:,:,1)=LL;

juvTime=sum(stageInts(t1H,1:end-1));
LL=diag([sj(t1H)*ones(1,juvTime) sa(t2H,1:totStage-juvTime-1)],-1);
LL(1,juvTime+1:juvTime+stageInts(t1H,end))=repro(t2H,1:stageInts(t1H,end));
L(:,:,2)=LL;

SADdist=[];
for j=1:2
	[SAD r]=eig(L(:,:,j));
	r=diag(r);
	rmax=max(abs(r));
	R=rmax^(1/timescale);

	SADdist1=SAD(:,find(abs(r)==rmax));
	SADdist1=SADdist1/sum(SADdist1);

	if figflag
		figure(103)
		subplot(2,1,j)
		semilogy(SADdist1)
		title(['R = ',num2str(R)])
	end

	SADdist=[SADdist SADdist1];
end

if tempset(3)==1
	mumInts=mumIntsbase;
else
	mumInts=flipud(mumIntsbase);
end

if tempset(4)==1
	atemp=atempbase;
else
	atemp=fliplr(atempbase);
end

% initial estimates
% MLL =
%   302.4948
est =  1.0e+02 * [0.023251489648733   0.000004673585631   0.015729465658049   0.003475426893859 ...
-0.000079553820714   0.006893463436913   0.004376599881524   0   0.007012662669429  ...
7.076364954197468   0.002169487437899   0.167601746433648     0  0  0.000208343799058];

est(8)=0.0001;

if superi==2
	est(13)=0.00001;
end
if superi==3
	est(14)=0.01;
end
if superi==4
	est(15)=0;
end

counter=0;
[ib,jb,startp]=find(est);
[mb,nb]=size(est);
MLL=meisnerfigfunct_20Aug13(startp)

% simulated annealing 
if 0
	estlist=[0 log(MLL) startp];
	for delta=[.02]
		counter=0;
		bcounter=0;

		fhandle=@meisnerfigfunct_20Aug13;
		annealingopts = struct('MaxTries',200,'InitTemp',.1,'StopTemp',10^-3,'Verbosity',0);
		[est MLL]=anneal(fhandle, startp,annealingopts);		

		startp=est;

		estlist=[estlist;delta log(MLL) est]
	end
end

if 1
	OPTIONS=optimset('MaxIter',10^5,'MaxFunEval',10^5,'TolX',10^-3,'TolFun',10^-3);
	counter=0;
	[est0 MLL]=fminsearch('meisnerfigfunct_20Aug13',startp,OPTIONS);

	est=full(sparse(ib,jb,est0,mb,nb));
end

meisnerfigfunct_20Aug13(est0)

% extract parameters
MLL
est

Q=length(Y)-8;
npar=length(est);
lnlike=-(Q/2)*log(2*pi)-MLL/2
AIC=-2*lnlike + 2*npar

alist=[est(1)*atemp est(1)*atemp]
k=est(2)
kp=est(3)*k
kk=est(4)
h=est(5)
sw=est(6)
s1=est(7)
s2=est(8)
s3=est(9)
rho=2/(1+exp(-est(10)))-1
mumDetect=est(11)
sm=est(12)

output=[lnlike est];

end
% Base parameter values
% counts =
%    1.0e+03 *
%     4.2070    0.1200    0.0000
% MLL =
%   302.4948
% est=  1.0e+02 * [0.023251489648733   0.000004673585631   0.015729465658049   0.003475426893859 ...
% -0.000079553820714   0.006893463436913   0.004376599881524   0   0.007012662669429  ...
% 7.076364954197468   0.002169487437899   0.167601746433648     0  0  0.000208343799058];
% lnlike =
%  -394.7661
% AIC =
%   821.5323
% alist =
%     0.8359    1.4893    0.8359    1.4893
% k =
%    4.6736e-04
% kp =
%    7.3513e-04
% kk =
%     0.3475
% h =
%    -0.0080
% sw =
%     0.6893
% s1 =
%     0.4377
% s2 =
%      0
% s3 =
%     0.7013
% rho =
%      1
% mumDetect =
%     0.2169
% sm =
%    16.7602

% LRTs
% 	-394.77	2.32E+00	4.67E-04	1.57E+00	3.48E-01	-7.96E-03	6.89E-01	4.38E-01	6.51E-05	7.01E-01	6.84E+02	2.17E-01	1.68E+01	0.00E+00	0.00E+00	2.08E-02
% 	-394.69	2.34E+00	4.52E-04	1.57E+00	3.49E-01	-8.90E-03	6.88E-01	4.37E-01	-6.82E-05	7.00E-01	1.85E+03	2.16E-01	1.68E+01	2.98E-05	0.00E+00	1.99E-02
% 	-393.80	2.45E+00	4.63E-04	1.56E+00	1.79E-01	-7.50E-03	6.87E-01	4.34E-01	-1.16E-03	8.28E-01	6.87E+03	1.96E-01	1.67E+01	0.00E+00	1.42E-01	2.20E-02
% 	-396.97	2.44E+00	4.67E-04	1.45E+00	2.87E-01	-1.85E-02	6.70E-01	4.37E-01	-2.30E-05	8.28E-01	1.48E+03	2.27E-01	1.76E+01	0.00E+00	0.00E+00	0.00E+00

% > 1-pchisq(2*(394.77-394.69),df=1)
% [1] 0.6891565
% > 1-pchisq(2*(394.77-393.80),df=1)
% [1] 0.1636685
% > 1-pchisq(2*(396.97-394.77),df=1)
% [1] 0.03593893
